@@ -1,4 +1,4 @@
-#' Table 1 for Health, Behavioral, and Social Scientists
+#' Table 1 for Simple and Stratified Descriptive Statistics
 #' 
 #' Produces a descriptive table, stratified by an optional categorical variable, 
 #' providing means/frequencies and standard deviations/percentages. 
@@ -15,53 +15,57 @@
 #' @param rounding the number of digits after the decimal; default is 3
 #' @param var_names custom variable names to be printed in the table
 #' @param format_output has three options: 1) "full" provides the table with the type of test, test statistic, and the p-value for each variable; 2) "pvalues" provides the table with the p-values; and 3) "stars" provides the table with stars indicating significance
-#' @param output_type default is "text"; the other option is "latex" which uses the \code{kable()} function in \code{knitr}
+#' @param output_type default is "text"; the other options are all format options in the \code{kable()} function in \code{knitr} (e.g., latex, html, markdown, pandoc)
 #' @param NAkeep when sset to \code{TRUE} it also shows how many missing values are in the data for each categorical variable being summarized
 #' @param m_label when \code{NAkeep = TRUE} this provides a label for the missing values in the table
-#' @param booktabs when \code{output_type = "latex"}; option is passed to \code{knitr::kable}
-#' @param caption when \code{output_type = "latex"}; option is passed to \code{knitr::kable}
-#' @param align when \code{output_type = "latex"}; option is passed to \code{knitr::kable}
+#' @param booktabs when \code{output_type != "text"}; option is passed to \code{knitr::kable}
+#' @param caption when \code{output_type != "text"}; option is passed to \code{knitr::kable}
+#' @param align when \code{output_type != "text"}; option is passed to \code{knitr::kable}
 #' 
 #' @return A table with the number of observations, means/frequencies and standard deviations/percentages is returned. The object is a \code{table1} class object with a print method. Can be printed in \code{LaTex} form.
 #'
 #' @examples 
-#' ## Data from MASS package ##
-#' library(MASS)
-#' data("birthwt")
-#' library(dplyr)
-#' b = mutate(.data=birthwt,
-#'            smoke = as.factor(smoke),
-#'            race  = as.factor(race),
-#'            ht    = as.factor(ht),
-#'            ui    = as.factor(ui))
-#' levels(b$race) = c("white", "black", "other")
-#' 
+#' ## Ficticious Data ##
 #' library(furniture)
+#' library(dplyr)
 #' 
-#' table1(b, age, race, smoke, ptl, ht, ui, ftv, NAkeep=TRUE)
-#' table1(b, age, race, smoke, ptl, ht, ui, ftv,
-#'        splitby=~factor(low),
-#'        NAkeep=TRUE)
-#'        
-#' b$low = as.factor(b$low)
-#' table1(b, age, race, smoke, ptl, ht, ui, ftv,
-#'        splitby=~low,
-#'        test=TRUE,
-#'        var_names = c("Age", "Race", "Smoking Status", "Previous Premature Labors", "Hypertension",
-#'                      "Uterine Irratibility", "Physician Visits"),
-#'        splitby_labels = c("Regular Birthweight", "Low Birthweight"))
-#'        
+#' x  <- runif(1000)
+#' y  <- rnorm(1000)
+#' z  <- factor(sample(c(0,1), 1000, replace=TRUE))
+#' a  <- factor(sample(c(1,2), 1000, replace=TRUE))
+#' df <- data.frame(x, y, z, a)
+#' 
+#' ## Simple
+#' table1(df, x, y, z, a)
+#' 
+#' ## Stratified
+#' ## both below are the same
+#' table1(df, x, y, z,
+#'        splitby = ~ a)
+#' table1(df, x, y, z,
+#'        splitby = "a")
+#' 
+#' ## With Piping
+#' df %>%
+#'   table1(x, y, z, 
+#'          splitby = ~a, 
+#'          piping = TRUE) %>%
+#'   summarise(count = n())
+#' 
+#' ## Adjust variables within function
+#' table1(df, ifelse(x > 0, 1, 0), z,
+#'        var_names = c("Dich X", "Z"))
+#'          
 #'
 #' @export
 #' @import stats
-#' @import lazyeval
 #' @importFrom knitr kable
-#' @importFrom car leveneTest
 table1 = function(.data, ..., splitby = NULL, splitby_labels = NULL, test = FALSE, test_type = "default", piping = FALSE,
                   rounding = 3, var_names = NULL, format_output = "pvalues", output_type = "text", NAkeep = FALSE, m_label = "Missing",
                   booktabs = TRUE, caption=NULL, align=NULL){
   
   # == # Checks and Data # == #
+  .call = match.call()
   
   if (NAkeep){ 
     NAkeep = "always" 
@@ -69,7 +73,7 @@ table1 = function(.data, ..., splitby = NULL, splitby_labels = NULL, test = FALS
     NAkeep = "no"
   }
   
-  data = table1_(.data, dots_capture(...))
+  data = table1_(..., d_=.data, .cl=.call)
   d = as.data.frame(data)
   
   ### Naming of variables
@@ -78,15 +82,16 @@ table1 = function(.data, ..., splitby = NULL, splitby_labels = NULL, test = FALS
     names(d) = var_names
   }
   
-  ### Splitby Variable
+  ### Splitby Variable (adds the variable to d as "split")
   if (is.null(splitby)){
     splitby_ = as.factor(1)
-    d$split = droplevels(splitby_)
+    d$split  = droplevels(splitby_)
   } else {
-    splitby_ = table1_(.data, splitby, split=TRUE)
-    d$split = droplevels(as.factor(splitby_[[1]]))
+    splitby_ = eval(parse(text = paste(splitby)[[length(paste(splitby))]]), .data)
+    d$split  = droplevels(as.factor(splitby_))
   }
   
+  ## Test = TRUE and the splitting variable needs to have more than one level
   if (test & length(levels(d$split))>1){
     test = TRUE
   } else {
@@ -103,12 +108,12 @@ table1 = function(.data, ..., splitby = NULL, splitby_labels = NULL, test = FALS
   tab = tab2 = tests = tests2 = nams = list()
   for (i in 1:(dim(d)[2]-1)){
     nams[[i]] = names(d)[i]
-    # If character
+    ## If character
     if (is.character(d[,i])){
-      d[,i] = as.factor(d[,i])
+      d[,i] = factor(d[,i])
     }
     
-    # If Factor
+    ## If Factor
     if (is.factor(d[,i])){
       tab[[i]] = tapply(d[,i], d$split, table, useNA=NAkeep)
       tab2[[i]] = tapply(d[,i], d$split, function(x) round(table(x, useNA=NAkeep)/sum(table(x, useNA=NAkeep)), rounding))
@@ -116,17 +121,21 @@ table1 = function(.data, ..., splitby = NULL, splitby_labels = NULL, test = FALS
         tests[[i]] = chisq.test(d$split, d[,i])
       if (test & test_type=="or")
         tests2[[i]] = glm(d$split ~ d[, i], family=binomial(link="logit"))
-      # If Numeric
+    ## If Numeric
     } else if (is.numeric(d[,i]) | is.integer(d[,i])){
       tab[[i]] = round(tapply(d[,i], d$split, mean, na.rm=TRUE), rounding)
       tab2[[i]] = round(tapply(d[,i], d$split, sd, na.rm=TRUE), rounding)
       if (length(levels(d$split))>2 & test){
-        lt = car::leveneTest(y=d[,i], group=d$split)$`Pr(>F)`[1]
+        ## Breusch-Pagan Test of Heteroskedasticity (equality of variances)
+        comp   = complete.cases(d[,i], d$split)
+        resids = resid(lm(d[comp,i] ~ d$split[comp]))^2
+        r2     = summary(lm(resids ~ nhanes$gender[comp]))$r.squared
+        lt     = dchisq(length(resids)*r2, df = length(levels(d$split)))
         if (lt<0.05){
-          # Performs an approximate method of Welch (1951)
+          ## Performs an approximate method of Welch (1951)
           tests[[i]] = oneway.test(d[,i] ~ d$split, var.equal=FALSE)
         } else {
-          # Performs a simple one-way ANOVA
+          ## Performs a simple one-way ANOVA
           tests[[i]] = oneway.test(d[,i] ~ d$split, var.equal=TRUE)
         }
       } else if (test){
@@ -137,7 +146,7 @@ table1 = function(.data, ..., splitby = NULL, splitby_labels = NULL, test = FALS
         tests2[[i]] = glm(d$split ~ d[, i], family=binomial(link="logit"))
       }
     } else {
-      paste("Variables need to be either factor or numeric.")
+      stop("Variables need to be either factor, character or numeric.", .call=FALSE)
     }
   }
   
@@ -159,7 +168,7 @@ table1 = function(.data, ..., splitby = NULL, splitby_labels = NULL, test = FALS
   
   for (j in 1:length(tab)){
     if (is.factor(d[,j])){
-      if (output_type == "latex"){
+      if (output_type != "text"){
         tabX = data.frame(paste("--  ", names(table(d[,j], useNA=NAkeep)), "  --"))
       } else {
         tabX = data.frame(paste("  ", names(table(d[,j], useNA=NAkeep))))
@@ -305,28 +314,27 @@ table1 = function(.data, ..., splitby = NULL, splitby_labels = NULL, test = FALS
     class(final_l) = c("table1", "list")
     if (piping){
       print(final_l)
-      invisible(data)
+      invisible(.data)
     } else {
       return(final_l)
     } 
-  } else if (output_type == "latex"){ ## latex compatible output from kable
+  } else if (output_type %in% c("latex", "markdown", "html", "pandoc", "rst")){ ##  output from kable
     if (piping){
-      knitr::kable(final, format="latex",
+      knitr::kable(final, format=output_type,
                    booktabs = booktabs,
                    caption = caption,
                    align = align,
                    row.names = FALSE)
       invisible(.data)
     } else {
-      knitr::kable(final, format="latex",
+      knitr::kable(final, format=output_type,
                    booktabs = booktabs,
                    caption = caption,
                    align = align,
                    row.names = FALSE)
-    } 
+    }
   }
 }
-citation("furniture")
 
 
 #' @export
@@ -350,58 +358,43 @@ print.table1 <- function(x, ...){
   cat("=====\n") 
 }
 
-#' Table 1 for Health, Behavioral, and Social Scientists
+#' Internal Table 1 Function
 #' 
-#' Produces a descriptive table, stratified by an optional categorical variable, 
-#' providing means/frequencies and standard deviations/percentages. 
-#' It is well-formatted for easy transition to academic article or report. 
-#' Can be used within the piping framework [see library(magrittr)].
+#' For internal use in table1().
 #' 
+#' @param ... the variables
 #' @param d_ the data.frame
-#' @param vars uses dots_capture() to capture the ...
-#' @param split for internal use in table1()
+#' @param .cl the original functon call
 #' 
 #' @return A data.frame
 #'
 #' @export
 #' @import stats
-#' @import lazyeval
-table1_ <- function(d_, vars, split=FALSE){
-  d1 = named = NULL
+table1_ <- function(..., d_, .cl=NULL){
+  df1 = df2 = NULL
+  vars = eval(substitute(alist(...)))
   
   ## for dots_capture
-  if (is.list(vars)){
-    for (i in seq_along(vars)){
-      named   <- paste(vars[[i]])
-      d1[[i]] <- f_eval(vars[[i]], d_)
-      
-      ## if is an index (built on assumption that lengths will differ)
-      if (length(d1[[i]]) != length(d_[[1]]) & is.numeric(d1[[i]])){
-        d2 <- d_[, d1[[i]]]
-        
-        ## if it is named vars
-      } else {
-        names(d1)[i] <- named[[2]]
-        d2 <- as.data.frame(d1)
-      }
-    }
+  for (i in seq_along(vars)){
+    df1[[i]] <- eval(vars[[i]], d_)
     
-    ## for single vars
-  } else if (split){
-    named   <- paste(vars)
-    d1      <- f_eval(vars, d_)
-    if (!is.factor(d1))
-      stop("'splitby' must be a formula of a factor variable (e.g. splitby = ~var1")
-    d2 <- as.data.frame(d1)
-    names(d2) <- named[[2]]
-  } 
+    ## if is an index (built on assumption that lengths will differ)
+    if (length(df1[[i]]) != length(d_[[1]]) & is.numeric(df1[[i]])){
+      df2 <- d_[, df1[[i]]]
+    }
+  }
   
+  if (is.null(df2)){
+    df2 <- as.data.frame(df1)
+    names(df2) = paste(.cl)[3:(length(vars)+2)]
+  }
+
   ## Error catching 
-  if (dim(d_)[1] != length(d2[[1]])){
+  if (dim(d_)[1] != length(df2[[1]])){
     stop("There is a problem with the variable names supplied. Make sure the ... only includes unquoted var names [e.g. gender, age] or a single vector of indices [e.g. c(3:5, 6)] or that splitby variable is stated as a formula [e.g. splitby = ~var1]",
          call.=FALSE)
   }
   
-  return(d2)
+  return(df2)
 }
 
